<?php

declare (strict_types=1);
namespace WpifyWooDeps\DI\Definition\Source;

use WpifyWooDeps\DI\Definition\ArrayDefinition;
use WpifyWooDeps\DI\Definition\AutowireDefinition;
use WpifyWooDeps\DI\Definition\DecoratorDefinition;
use WpifyWooDeps\DI\Definition\Definition;
use WpifyWooDeps\DI\Definition\Exception\InvalidDefinition;
use WpifyWooDeps\DI\Definition\FactoryDefinition;
use WpifyWooDeps\DI\Definition\Helper\DefinitionHelper;
use WpifyWooDeps\DI\Definition\ObjectDefinition;
use WpifyWooDeps\DI\Definition\ValueDefinition;
/**
 * Turns raw definitions/definition helpers into definitions ready
 * to be resolved or compiled.
 *
 * @author Matthieu Napoli <matthieu@mnapoli.fr>
 */
class DefinitionNormalizer
{
    /**
     * @var Autowiring
     */
    private $autowiring;
    public function __construct(\WpifyWooDeps\DI\Definition\Source\Autowiring $autowiring)
    {
        $this->autowiring = $autowiring;
    }
    /**
     * Normalize a definition that is *not* nested in another one.
     *
     * This is usually a definition declared at the root of a definition array.
     *
     * @param mixed $definition
     * @param string $name The definition name.
     * @param string[] $wildcardsReplacements Replacements for wildcard definitions.
     *
     * @throws InvalidDefinition
     */
    public function normalizeRootDefinition($definition, string $name, array $wildcardsReplacements = null) : \WpifyWooDeps\DI\Definition\Definition
    {
        if ($definition instanceof \WpifyWooDeps\DI\Definition\Helper\DefinitionHelper) {
            $definition = $definition->getDefinition($name);
        } elseif (\is_array($definition)) {
            $definition = new \WpifyWooDeps\DI\Definition\ArrayDefinition($definition);
        } elseif ($definition instanceof \Closure) {
            $definition = new \WpifyWooDeps\DI\Definition\FactoryDefinition($name, $definition);
        } elseif (!$definition instanceof \WpifyWooDeps\DI\Definition\Definition) {
            $definition = new \WpifyWooDeps\DI\Definition\ValueDefinition($definition);
        }
        // For a class definition, we replace * in the class name with the matches
        // *Interface -> *Impl => FooInterface -> FooImpl
        if ($wildcardsReplacements && $definition instanceof \WpifyWooDeps\DI\Definition\ObjectDefinition) {
            $definition->replaceWildcards($wildcardsReplacements);
        }
        if ($definition instanceof \WpifyWooDeps\DI\Definition\AutowireDefinition) {
            $definition = $this->autowiring->autowire($name, $definition);
        }
        $definition->setName($name);
        try {
            $definition->replaceNestedDefinitions([$this, 'normalizeNestedDefinition']);
        } catch (\WpifyWooDeps\DI\Definition\Exception\InvalidDefinition $e) {
            throw \WpifyWooDeps\DI\Definition\Exception\InvalidDefinition::create($definition, \sprintf('Definition "%s" contains an error: %s', $definition->getName(), $e->getMessage()), $e);
        }
        return $definition;
    }
    /**
     * Normalize a definition that is nested in another one.
     *
     * @param mixed $definition
     * @return mixed
     *
     * @throws InvalidDefinition
     */
    public function normalizeNestedDefinition($definition)
    {
        $name = '<nested definition>';
        if ($definition instanceof \WpifyWooDeps\DI\Definition\Helper\DefinitionHelper) {
            $definition = $definition->getDefinition($name);
        } elseif (\is_array($definition)) {
            $definition = new \WpifyWooDeps\DI\Definition\ArrayDefinition($definition);
        } elseif ($definition instanceof \Closure) {
            $definition = new \WpifyWooDeps\DI\Definition\FactoryDefinition($name, $definition);
        }
        if ($definition instanceof \WpifyWooDeps\DI\Definition\DecoratorDefinition) {
            throw new \WpifyWooDeps\DI\Definition\Exception\InvalidDefinition('Decorators cannot be nested in another definition');
        }
        if ($definition instanceof \WpifyWooDeps\DI\Definition\AutowireDefinition) {
            $definition = $this->autowiring->autowire($name, $definition);
        }
        if ($definition instanceof \WpifyWooDeps\DI\Definition\Definition) {
            $definition->setName($name);
            // Recursively traverse nested definitions
            $definition->replaceNestedDefinitions([$this, 'normalizeNestedDefinition']);
        }
        return $definition;
    }
}
